//
//  ArchitectureLoader.swift
//  iOS Architectures
//
//  Created by Dave DeLong on 10/27/17.
//  Copyright © 2017 Dave DeLong. All rights reserved.
//

import Foundation

// A simple architecture loader that hits iosarchitecture.top

class ArchitectureLoader: ArchitectureLoading {
    private let session: URLSession
    private let decoder = JSONDecoder()
    
    init() {
        let configuration = URLSessionConfiguration.default
        configuration.timeoutIntervalForRequest = 2
        configuration.timeoutIntervalForResource = 2
        session = URLSession(configuration: configuration, delegate: nil, delegateQueue: .main)
    }
    
    func loadArchitecture(_ completion: @escaping (Result<Architecture>) -> Void) {
        let url = URL(string: "https://iosarchitecture.top/architecture")!
        let task = session.dataTask(with: url) { (data, response, error) in
            if let error = error { return completion(.failure(error.localizedDescription)) }
            
            if let data = data {
                do {
                    let architecture = try self.decoder.decode(Architecture.self, from: data)
                    return completion(.success(architecture))
                    
                } catch let e {
                    return completion(.failure(e.localizedDescription))
                }
            } else {
                return completion(.failure("Invalid data response"))
            }
        }
        task.resume()
    }
    
    
    
}
