//
//  GenerateArchitectureViewController.swift
//  iOS Architectures
//
//  Created by Dave DeLong on 10/27/17.
//  Copyright © 2017 Dave DeLong. All rights reserved.
//

import UIKit

/// The primary "flow" view controller for the app.
/// It has no UI of its own, but instead contains the simple logic
/// to decide which child view controller should be shown
class GenerateArchitectureViewController: UIViewController {

    let loader: ArchitectureLoading
    
    init(loader: ArchitectureLoading) {
        self.loader = loader
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) { fatalError("die") }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // as the view loads, we want to start with a welcome message
        let start = MessageViewController(message: "Generate an iOS app architecture", buttonLabel: "Go!")
        start.delegate = self
        
        // this is a custom UIViewController method
        // it's implementation is in UIViewController+SimpleTransition.swift
        transition(to: start)
    }
    
    /// Load a new architecture
    func loadAnotherArchitecture() {
        // first, we'll transition to the loading state, which is its own view controller
        transition(to: LoadingViewController()) { _ in
            
            // after the transition completes, we'll request a new architecture
            self.loader.loadArchitecture { r in
                
                // when the loading is complete...
                switch r {
                    
                case .failure(let e):
                    // we'll show an error message if something went wrong
                    let error = MessageViewController(message: e)
                    error.delegate = self
                    self.transition(to: error)
                    
                case .success(let a):
                    // we'll show the architecture if we got the value
                    let arch = ArchitectureViewController(architecture: a)
                    arch.delegate = self
                    self.transition(to: arch)
                }
            }
        }
    }
}
    
extension GenerateArchitectureViewController: MessageViewControllerDelegate, ArchitectureViewControllerDelegate {
    
    /// a MessageViewController has indicated we should fetch a new architecture
    func messageViewControllerActionButtonWasTapped(_ messageVC: MessageViewController) {
        loadAnotherArchitecture()
    }
    
    /// an ArchitectureViewController has indicated we should fetch a new architecture
    func architectureViewControllerActionButtonWasTapped(_ sender: ArchitectureViewController) {
        loadAnotherArchitecture()
    }

}
